
C------------------------------------------------------------------------
C Determine slit width (Global_sw) in pixels using flat field image.
C z and b are scratch arrays.

      subroutine EE_FindSlitWidth(sc,ec,sr,er,flat)

      implicit none
      integer*4 sc,ec,sr,er            ! image dimensions (input)
      real*4 flat(sc:ec,sr:er)         ! flat field image (input)
C
      include 'makee.inc'
      include 'global.inc'
      include 'scratch.inc'
      include 'soun.inc'
C
      integer*4 ii,cc,row,col,i,j,k,narr,pkk,k2,num,newk
      real*4 dropfrac,rms,sum,wsum,pkval

C Initialize.
      dropfrac = 0.90;

C Try again (26mar2012).
2012  continue

C Initialize.
      pkk = 0
      k2  = 0
      pkval=0.
      narr= 0
      Global_sw = 0.

C Find slit width for several regions on image.
      DO cc=sc+200,ec-200,200

C Make vertical mash.
        do row=sr,er
          z(row) = 0.
          do col=cc-20,cc+20
            z(row) = z(row) + flat(col,row)
          enddo
          z(row) = z(row) / 21.
        enddo

C Correlation function.
        do k=1,er-sr
          b(k)= 0.
        enddo
        do i=sr,er
          do j=i+1,er
            k=j-i
            b(k) = b(k) + abs(z(i)-z(j))
          enddo
        enddo

C Determine first peak.
        k=5 
        do while(k.lt.er-sr)
          if (b(k).lt.b(k-1)) then
            pkk  = k-1
            pkval= b(k-1)
            k=er-sr
          else
            k=k+1
          endif
        enddo

C Find point where it drops by 'dropfrac'.
        k=pkk
        do while(k.lt.er-sr)
          if (b(k).lt.pkval*dropfrac) then
            k2 = k-1
            k  = er-sr
          else
            k=k+1
          endif
        enddo

C Now find centroid.
        sum =0.
        wsum=0.
        num =0
        do k=1,k2
          if (b(k).gt.pkval*dropfrac) then
            sum = sum + float(k)*(b(k)-(pkval*dropfrac))
            wsum=wsum + (b(k)-(pkval*dropfrac))
            num = num + 1
          endif
        enddo

        if (wsum.gt.1.e-20) then
          narr = narr + 1
          arr(narr) = sum / wsum

C :::Code2005:::
          if (Global_HIRES2) then
C Now back up and find the lowest point that is within 2% of peak...
            k    = nint(( arr(narr) ))
            newk = k
            k    = k - 1
            do while(k.gt.0)
              if (b(k).lt.(pkval*0.98)) then
                newk = k
                k = -1
              endif
              k = k - 1
            enddo
            arr(narr) = newk + 1
          endif
C :::Code2005:::

        endif
   
      ENDDO

C Check.
      if (narr.lt.3) then
        write(soun,'(a,i6)') 
     .  'ERROR: EE_FindSlitWidth: Estimating slit width.  narr=',narr
        call exit(1)
      endif
      call find_median(arr,narr,Global_sw)

C Look at RMS in arr values. (26 March 2012).
      rms = 0.
      do ii=1,narr
        rms = rms + ( (arr(ii) - Global_sw) * (arr(ii) - Global_sw) )
      enddo
      rms = sqrt( rms / float(narr) )
      write(soun,'(a,f9.4,a,f9.4,a,f9.4,a)') 
     .  'NOTE:(26mar2012):sw=',Global_sw,' rms=',
     .  rms,' [',Global_sw/rms,']'
      if (((Global_sw/rms).lt.9.).and.(dropfrac.lt.0.91)) then
        write(soun,'(2a)') 'warning: found possible slit problem.',
     .                   '... try again with different threshold.'
        dropfrac = 0.97
        goto 2012
      endif

C Slight correction.
      Global_sw = Global_sw * 1.1
      return
      end

